@@ -22,6 +22,10 @@
  *     Mark McLoughlin <mark@skynet.ie>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -30,6 +34,7 @@
 #include <bonobo/bonobo-property-bag.h>
 #include <bonobo/bonobo-item-handler.h>
 #include <bonobo/bonobo-shlib-factory.h>
+#include <bonobo/bonobo-property-bag-client.h>
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <gconf/gconf.h>
@@ -44,6 +49,7 @@
 struct _PanelAppletPrivate {
 	PanelAppletShell           *shell;
 	BonoboControl              *control;
+	BonoboPropertyBag          *prop_sack;
 	BonoboItemHandler          *item_handler;
 
 	gchar                      *iid;
@@ -51,13 +57,11 @@ struct _PanelAppletPrivate {
 	gboolean                    bound;
 	gchar                      *prefs_key;
 
+	PanelAppletFlags            flags;
 	PanelAppletOrient           orient;
 	guint                       size;
 	gchar                      *background;
 
-	gboolean                    expand_major;
-	gboolean                    expand_minor;
-
 	GdkPixmap                  *bg_pixmap;
 };
 
@@ -76,22 +80,24 @@ static guint panel_applet_signals [LAST_SIGNAL];
 #define PROPERTY_ORIENT     "panel-applet-orient"
 #define PROPERTY_SIZE       "panel-applet-size"
 #define PROPERTY_BACKGROUND "panel-applet-background"
+#define PROPERTY_FLAGS      "panel-applet-flags"
 
 enum {
 	PROPERTY_ORIENT_IDX,
 	PROPERTY_SIZE_IDX,
-	PROPERTY_BACKGROUND_IDX
+	PROPERTY_BACKGROUND_IDX,
+	PROPERTY_FLAGS_IDX,
 };
 
 static void
-panel_applet_associate_schemas_in_dir (GConfEngine  *engine,
+panel_applet_associate_schemas_in_dir (GConfClient  *client,
 				       const gchar  *prefs_key,
 				       const gchar  *schema_dir,
 				       GError      **error)
 {
 	GSList *list, *l;
 
-	list = gconf_engine_all_entries (engine, schema_dir, error);
+	list = gconf_client_all_entries (client, schema_dir, error);
 
 	g_return_if_fail (list != NULL);
 	g_return_if_fail (*error == NULL);
@@ -107,7 +113,8 @@ panel_applet_associate_schemas_in_dir (GConfEngine  *engine,
 
 		g_free (tmp);
 
-		gconf_engine_associate_schema (engine, key, gconf_entry_get_key (entry), error);
+		gconf_engine_associate_schema (
+			client->engine, key, gconf_entry_get_key (entry), error);
 
 		g_free (key);
 
@@ -121,7 +128,7 @@ panel_applet_associate_schemas_in_dir (GConfEngine  *engine,
 
 	g_slist_free (list);
 
-	list = gconf_engine_all_dirs (engine, schema_dir, error);
+	list = gconf_client_all_dirs (client, schema_dir, error);
 
 	for (l = list; l; l = l->next) {
 		gchar *subdir = l->data;
@@ -131,7 +138,8 @@ panel_applet_associate_schemas_in_dir (GConfEngine  *engine,
 		prefs_subdir  = g_strdup_printf ("%s/%s", prefs_key, subdir);
 		schema_subdir = g_strdup_printf ("%s/%s", schema_dir, subdir);
 
-		panel_applet_associate_schemas_in_dir (engine, prefs_subdir, schema_subdir, error);
+		panel_applet_associate_schemas_in_dir (
+			client, prefs_subdir, schema_subdir, error);
 
 		g_free (prefs_subdir);
 		g_free (schema_subdir);
@@ -151,7 +159,6 @@ panel_applet_add_preferences (PanelApplet  *applet,
 			      const gchar  *schema_dir,
 			      GError      **opt_error)
 {
-	GConfEngine  *engine;
 	GConfClient  *client;
 	GError      **error = NULL;
 	GError       *our_error = NULL;
@@ -163,12 +170,10 @@ panel_applet_add_preferences (PanelApplet  *applet,
 	else
 		error = &our_error;
 
-	engine = gconf_engine_get_default ();
+	client = gconf_client_get_default ();
 
-	panel_applet_associate_schemas_in_dir (engine,
-					       applet->priv->prefs_key,
-					       schema_dir,
-					       error);
+	panel_applet_associate_schemas_in_dir (
+		client, applet->priv->prefs_key, schema_dir, error);
 
 	if (!opt_error && our_error) {
 		g_warning (G_STRLOC ": failed to add preferences from '%s' : '%s'",
@@ -176,12 +181,8 @@ panel_applet_add_preferences (PanelApplet  *applet,
 		g_error_free (our_error);
 	}
 
-	client = gconf_client_get_default ();
-
-	gconf_client_add_dir (client,
-			      applet->priv->prefs_key,
-			      GCONF_CLIENT_PRELOAD_NONE,
-			      NULL);
+	gconf_client_add_dir (
+		client, applet->priv->prefs_key, GCONF_CLIENT_PRELOAD_NONE, NULL);
 }
 
 gchar *
@@ -195,26 +196,21 @@ panel_applet_get_preferences_key (PanelApplet *applet)
 	return g_strdup (applet->priv->prefs_key);
 }
 
-void
-panel_applet_get_expand_flags (PanelApplet *applet,
-			       gboolean    *expand_major,
-			       gboolean    *expand_minor)
+PanelAppletFlags
+panel_applet_get_flags (PanelApplet *applet)
 {
-	g_return_if_fail (PANEL_IS_APPLET (applet));
+	g_return_val_if_fail (PANEL_IS_APPLET (applet), PANEL_APPLET_FLAGS_NONE);
 
-	*expand_major = applet->priv->expand_major;
-	*expand_minor = applet->priv->expand_minor;
+	return applet->priv->flags;
 }
 
 void
-panel_applet_set_expand_flags (PanelApplet *applet,
-			       gboolean     expand_major,
-			       gboolean     expand_minor)
+panel_applet_set_flags (PanelApplet      *applet,
+			PanelAppletFlags  flags)
 {
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 
-	applet->priv->expand_major = expand_major;
-	applet->priv->expand_minor = expand_minor;
+	applet->priv->flags = flags;
 }
 
 /**
@@ -310,7 +306,7 @@ panel_applet_setup_menu_from_file (PanelApplet        *applet,
 	g_return_if_fail (file != NULL && verb_list != NULL);
 
 	if (!opt_datadir)
-		opt_datadir = GNOME_DATADIR;
+		opt_datadir = PANEL_APPLET_DATADIR;
 
 	if (!opt_app_name)
 		opt_app_name = app_name = g_strdup_printf ("%d", getpid ());
@@ -378,6 +374,36 @@ panel_applet_finalize (GObject *object)
 	parent_class->finalize (object);
 }
 
+static void
+panel_applet_menu_position (GtkMenu  *menu,
+			    gint     *x,
+			    gint     *y,
+			    gboolean *push_in,
+			    gpointer  data)
+{
+	GtkWidget *w = data;
+	gint wx, wy;
+
+	g_return_if_fail (w != NULL);
+
+	gdk_window_get_origin (w->window, &wx, &wy);
+	/*
+         * Make sure that the popup position is in the panel
+	 * as the menu may be popped up by a keystroke
+	 */
+	if (*x < wx)
+		*x = wx;
+	else if (*x > wx + w->allocation.width)
+		*x = wx + w->allocation.width;
+
+	if (*y < wy)
+		*y = wy;
+	 else if (*y > wy + w->allocation.height)
+		*y = wy + w->allocation.height;
+
+	*push_in = TRUE;
+}
+
 static gboolean
 panel_applet_button_press (GtkWidget      *widget,
 			   GdkEventButton *event)
@@ -385,7 +411,7 @@ panel_applet_button_press (GtkWidget      *widget,
 	PanelApplet *applet = PANEL_APPLET (widget);
 
 	if (event->button == 3) {
-		bonobo_control_do_popup (applet->priv->control,
+		bonobo_control_do_popup (applet->priv->control, 
 					 event->button,
 					 event->time);
 		return TRUE;
@@ -394,9 +420,64 @@ panel_applet_button_press (GtkWidget      *widget,
 	return FALSE;
 }
 
+static gboolean
+panel_applet_popup_menu (GtkWidget *widget)
+{
+	PanelApplet *applet = PANEL_APPLET (widget);
+
+	bonobo_control_do_popup_full (applet->priv->control, NULL, NULL,
+				      panel_applet_menu_position, widget,
+				      3,
+				      GDK_CURRENT_TIME);
+	return TRUE;
+	
+}
+
+static gboolean
+panel_applet_expose (GtkWidget      *widget,
+		     GdkEventExpose *event) 
+{
+	g_return_val_if_fail (PANEL_IS_APPLET (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+
+	GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+
+        if (GTK_WIDGET_HAS_FOCUS (widget))
+		gtk_paint_focus (widget->style, widget->window,
+                                 GTK_WIDGET_STATE (widget),
+                                 &event->area, widget, "panel_applet",
+                                 widget->allocation.x + 1,
+                                 widget->allocation.y + 1,
+                                 widget->allocation.width - 3,
+                                 widget->allocation.height - 3);
+
+	return FALSE;
+}                
+
+static gboolean 
+panel_applet_focus (GtkWidget        *widget,
+		    GtkDirectionType  dir)
+{
+	gboolean ret;
+
+	ret = GTK_WIDGET_CLASS (parent_class)->focus (widget, dir);
+	if (!ret) {
+ 		if (!GTK_CONTAINER (widget)->focus_child)  {
+			/*
+			 * Applet does not have a widget which can focus so set
+			 * the focus on the applet.
+			 */ 
+			GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
+			gtk_widget_grab_focus (widget);
+		}
+	}
+
+	return TRUE;
+}
+
 static gboolean
 panel_applet_parse_color (const gchar *color_str,
-			   GdkColor    *color)
+			  GdkColor    *color)
 {
 	int r, g, b;
 
@@ -502,7 +583,89 @@ panel_applet_get_pixmap (PanelApplet     *applet,
 
 	return retval;
 }
-			
+
+static PanelAppletBackgroundType
+panel_applet_handle_background_string (PanelApplet  *applet,
+				       GdkColor     *color,
+				       GdkPixmap   **pixmap)
+{
+	gchar **elements;
+
+	if (!applet->priv->background)
+		return PANEL_NO_BACKGROUND;
+
+	elements = g_strsplit (applet->priv->background, ":", -1);
+
+	if (elements [0] && !strcmp (elements [0], "none" )) {
+		return PANEL_NO_BACKGROUND;
+		
+	} else if (elements [0] && !strcmp (elements [0], "color")) {
+		g_return_val_if_fail (color != NULL, PANEL_NO_BACKGROUND);
+
+		if (!elements [1] || !panel_applet_parse_color (elements [1], color)) {
+
+			g_warning (_("Incomplete '%s' background type received"), elements [0]);
+			g_strfreev (elements);
+			return PANEL_NO_BACKGROUND;
+		}
+
+		return PANEL_COLOR_BACKGROUND;
+
+	} else if (elements [0] && !strcmp (elements [0], "pixmap")) {
+		GdkNativeWindow pixmap_id;
+		int             x, y;
+
+		g_return_val_if_fail (pixmap != NULL, PANEL_NO_BACKGROUND);
+
+		if (!panel_applet_parse_pixmap_str (elements [1], &pixmap_id, &x, &y)) {
+			g_warning (_("Incomplete '%s' background type received: %s"),
+				   elements [0], elements [1]);
+
+			g_strfreev (elements);
+			return PANEL_NO_BACKGROUND;
+		}
+
+		*pixmap = panel_applet_get_pixmap (applet, pixmap_id, x, y);
+		if (!*pixmap) {
+			g_warning (_("Failed to get pixmap %s"), elements [1]);
+			g_strfreev (elements);
+			return PANEL_NO_BACKGROUND;
+		}
+
+		return PANEL_PIXMAP_BACKGROUND;
+	} else
+		g_warning (_("Unknown background type received"));
+
+	return PANEL_NO_BACKGROUND;
+}
+
+
+/**
+ * panel_applet_get_background
+ * @applet: A #PanelApplet.
+ * @color: A #GdkColor to be filled in.
+ * @pixmap: Returned #GdkPixmap
+ *
+ * Returns the current background type and fills in the relevant
+ * 
+ * Return value: a #PanelAppletOrient value.
+ */
+PanelAppletBackgroundType
+panel_applet_get_background (PanelApplet *applet,
+			     GdkColor *color,
+			     GdkPixmap **pixmap)
+{
+	g_return_val_if_fail (PANEL_IS_APPLET (applet), PANEL_NO_BACKGROUND);
+
+	/* initial sanity */
+	if (pixmap != NULL)
+		*pixmap = NULL;
+	if (color != NULL)
+		memset (color, 0, sizeof (GdkColor));
+
+	return panel_applet_handle_background_string (applet, color, pixmap);
+}
+
 static void
 panel_applet_get_prop (BonoboPropertyBag *sack,
                        BonoboArg         *arg,
@@ -522,6 +685,9 @@ panel_applet_get_prop (BonoboPropertyBag *sack,
 	case PROPERTY_BACKGROUND_IDX:
 		BONOBO_ARG_SET_STRING (arg, applet->priv->background);
 		break;
+	case PROPERTY_FLAGS_IDX:
+		BONOBO_ARG_SET_SHORT (arg, applet->priv->flags);
+		break;
 	default:
 		g_assert_not_reached ();
 		break;
@@ -567,80 +733,40 @@ panel_applet_set_prop (BonoboPropertyBag *sack,
 		}
 		break;
 	case PROPERTY_BACKGROUND_IDX: {
-		gchar  *bg_str;
-		gchar **elements;
+		PanelAppletBackgroundType  type;
+		GdkColor                   color;
+		GdkPixmap                 *pixmap = NULL;
 
-		bg_str = BONOBO_ARG_GET_STRING (arg);
 
-		elements = g_strsplit (bg_str, ":", -1);
+		if (applet->priv->background)
+			g_free (applet->priv->background);
 
-		if (elements [0] && !strcmp (elements [0], "none" )) {
-			if (applet->priv->background)
-				g_free (applet->priv->background);
-			applet->priv->background = NULL;
+		applet->priv->background = g_strdup (BONOBO_ARG_GET_STRING (arg));
 
+		type = panel_applet_handle_background_string (applet, &color, &pixmap);
+
+		switch (type) {
+		case PANEL_NO_BACKGROUND:
 			g_signal_emit (G_OBJECT (applet),
 				       panel_applet_signals [CHANGE_BACKGROUND],
 				       0, PANEL_NO_BACKGROUND, NULL, NULL);
-		
-		} else if (elements [0] && !strcmp (elements [0], "color")) {
-			GdkColor color;
-
-			if (!elements [1] || !panel_applet_parse_color (elements [1], &color)) {
-
-				g_warning (_("panel_applet_set_prop: Incomplete '%s'"
-					     " background type received"), elements [0]);
-
-				g_strfreev (elements);
-				return;
-			}
-
-			if (applet->priv->background)
-				g_free (applet->priv->background);
-			applet->priv->background = g_strdup (bg_str);
-
+			break;
+		case PANEL_COLOR_BACKGROUND:
 			g_signal_emit (G_OBJECT (applet),
 				       panel_applet_signals [CHANGE_BACKGROUND],
 				       0, PANEL_COLOR_BACKGROUND, &color, NULL);
-
-		} else if (elements [0] && !strcmp (elements [0], "pixmap")) {
-			GdkNativeWindow  pixmap_id;
-			GdkPixmap       *pixmap;
-			int              x, y;
-
-			if (!panel_applet_parse_pixmap_str (elements [1], &pixmap_id, &x, &y)) {
-				g_warning (_("panel_applet_set_prop: Incomplete '%s'"
-					     " background type received: %s"),
-					   elements [0], elements [1]);
-
-				g_strfreev (elements);
-				return;
-			}
-
-			pixmap = panel_applet_get_pixmap (applet, pixmap_id, x, y);
-			if (!pixmap) {
-				g_warning (_("panel_applet_set_prop: Failed to get pixmap"
-					     " %s"), elements [1]);
-
-				g_strfreev (elements);
-				return;
-			}
-
-			if (applet->priv->background)
-				g_free (applet->priv->background);
-			applet->priv->background = g_strdup (bg_str);
-
+			break;
+		case PANEL_PIXMAP_BACKGROUND:
 			g_signal_emit (G_OBJECT (applet),
 				       panel_applet_signals [CHANGE_BACKGROUND],
 				       0, PANEL_PIXMAP_BACKGROUND, NULL, pixmap);
 
 			g_object_unref (G_OBJECT (pixmap));
-		} else {
-			g_warning (_("panel_applet_set_prop: Unknown background type received"));
-			return;
+			break;
+		default:
+			g_assert_not_reached ();
+			break;
 		}
-
-		g_strfreev (elements);
 		}
 		break;
 	default:
@@ -682,6 +808,14 @@ panel_applet_property_bag (PanelApplet *applet)
 				 _("The Applet's containing Panel's background color or pixmap"),
 				 Bonobo_PROPERTY_READABLE | Bonobo_PROPERTY_WRITEABLE);
 
+	bonobo_property_bag_add (sack,
+				 PROPERTY_FLAGS,
+				 PROPERTY_FLAGS_IDX,
+				 BONOBO_ARG_SHORT,
+				 NULL,
+				 _("The Applet's flags"),
+				 Bonobo_PROPERTY_READABLE);
+
 	return sack;
 }
 
@@ -731,16 +865,80 @@ panel_applet_item_handler_get_object (BonoboItemHandler *handler,
 	for (l = options; l; l = l->next) {
 		BonoboItemOption *option = l->data;
 
-		if (!strcmp (option->key, "prefs_key") && !applet->priv->prefs_key &&
-		    option->value && option->value [0])
+		if (!option->value || !option->value [0])
+			continue;
+
+		if (!strcmp (option->key, "prefs_key") && !applet->priv->prefs_key)
 			applet->priv->prefs_key = g_strdup (option->value);
+
+		else if (!strcmp (option->key, "background"))
+			bonobo_pbclient_set_string (BONOBO_OBJREF (applet->priv->prop_sack),
+						    PROPERTY_BACKGROUND, option->value, NULL);
+
+		else if (!strcmp (option->key, "orient")) {
+			if (!strcmp (option->value, "up"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_ORIENT,
+					PANEL_APPLET_ORIENT_UP, NULL);
+
+			else if (!strcmp (option->value, "down"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_ORIENT,
+					PANEL_APPLET_ORIENT_DOWN, NULL);
+
+			else if (!strcmp (option->value, "left"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_ORIENT,
+					PANEL_APPLET_ORIENT_LEFT, NULL);
+
+			else if (!strcmp (option->value, "right"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_ORIENT,
+					PANEL_APPLET_ORIENT_RIGHT, NULL);
+
+		} else if (!strcmp (option->key, "size")) {
+			if (!strcmp (option->value, "xx-small"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_SIZE,
+					GNOME_Vertigo_PANEL_XX_SMALL, NULL);
+
+			else if (!strcmp (option->value, "x-small"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_SIZE,
+					GNOME_Vertigo_PANEL_X_SMALL, NULL);
+
+			else if (!strcmp (option->value, "small"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_SIZE,
+					GNOME_Vertigo_PANEL_SMALL, NULL);
+
+			else if (!strcmp (option->value, "medium"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_SIZE,
+					GNOME_Vertigo_PANEL_MEDIUM, NULL);
+
+			else if (!strcmp (option->value, "large"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_SIZE,
+					GNOME_Vertigo_PANEL_LARGE, NULL);
+
+			else if (!strcmp (option->value, "x-large"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_SIZE,
+					GNOME_Vertigo_PANEL_X_LARGE, NULL);
+
+			else if (!strcmp (option->value, "xx-large"))
+				bonobo_pbclient_set_short (
+					BONOBO_OBJREF (applet->priv->prop_sack), PROPERTY_SIZE,
+					GNOME_Vertigo_PANEL_XX_LARGE, NULL);
+		}
 	}
 
 	bonobo_item_options_free (options);
 
 	return bonobo_object_dup_ref (BONOBO_OBJREF (applet->priv->control), ev);
 }
-			   
+
 static void
 panel_applet_class_init (PanelAppletClass *klass,
 			 gpointer          dummy)
@@ -751,6 +949,8 @@ panel_applet_class_init (PanelAppletClass *klass,
 	parent_class = g_type_class_peek_parent (klass);
 
 	widget_class->button_press_event = panel_applet_button_press;
+	widget_class->expose_event = panel_applet_expose;
+	widget_class->focus = panel_applet_focus;
 
 	gobject_class->finalize = panel_applet_finalize;
 
@@ -801,12 +1001,10 @@ panel_applet_instance_init (PanelApplet      *applet,
 
 	applet->priv->bound        = FALSE;
 
+	applet->priv->flags        = PANEL_APPLET_FLAGS_NONE;
 	applet->priv->orient       = PANEL_APPLET_ORIENT_UP;
 	applet->priv->size         = GNOME_Vertigo_PANEL_MEDIUM;
 
-	applet->priv->expand_major = FALSE;
-	applet->priv->expand_minor = FALSE;
-
 	gtk_widget_set_events (GTK_WIDGET (applet), 
 			       GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 }
@@ -837,8 +1035,8 @@ panel_applet_get_type (void)
 	return type;
 }
 
-void
-panel_applet_construct (PanelApplet *applet)
+static void
+panel_applet_setup (PanelApplet *applet)
 {
 	PanelAppletPrivate *priv;
 
@@ -851,21 +1049,25 @@ panel_applet_construct (PanelApplet *applet)
 			  G_CALLBACK (panel_applet_control_bound),
 			  applet);
 
-	bonobo_control_set_properties (priv->control,
-				       BONOBO_OBJREF (panel_applet_property_bag (applet)),
-				       NULL);
+	priv->prop_sack = panel_applet_property_bag (applet);
+
+	bonobo_control_set_properties (
+			priv->control, BONOBO_OBJREF (priv->prop_sack), NULL);
 
 	priv->shell = panel_applet_shell_new (applet);
 
 	bonobo_object_add_interface (BONOBO_OBJECT (priv->control),
 				     BONOBO_OBJECT (priv->shell));
 
-	priv->item_handler = bonobo_item_handler_new (NULL,
-						      panel_applet_item_handler_get_object,
-						      applet);
+	priv->item_handler = 
+		bonobo_item_handler_new (
+			NULL, panel_applet_item_handler_get_object, applet);
 
 	bonobo_object_add_interface (BONOBO_OBJECT (priv->control),
 				     BONOBO_OBJECT (priv->item_handler));
+
+	g_signal_connect (G_OBJECT (applet), "popup_menu",
+			  G_CALLBACK (panel_applet_popup_menu), NULL);
 }
 
 /**
@@ -884,22 +1086,50 @@ panel_applet_new (void)
 
 	applet = g_object_new (PANEL_TYPE_APPLET, NULL);
 
-	panel_applet_construct (applet);
+	panel_applet_setup (applet);
 
 	return GTK_WIDGET (applet);
 }
 
-static BonoboObject *
-panel_applet_factory_callback (BonoboGenericFactory *factory,
-			       const char           *iid,
-			       gpointer              user_data)
-{
-	PanelApplet        *applet;
+typedef struct {
+	GType      applet_type;
+	GClosure *closure;
+} PanelAppletCallBackData;
 
-	applet = PANEL_APPLET (panel_applet_new ());
+static PanelAppletCallBackData *
+panel_applet_callback_data_new (GType     applet_type,
+				GClosure *closure)
+{
+	PanelAppletCallBackData *retval;
+
+	retval = g_new0 (PanelAppletCallBackData, 1);
+
+	retval->applet_type = applet_type;
+	retval->closure     = closure;
+
+	return retval;
+}
+
+static void
+panel_applet_callback_data_free (PanelAppletCallBackData *data)
+{
+	g_closure_unref (data->closure);
+	g_free (data);
+}
+
+static BonoboObject *
+panel_applet_factory_callback (BonoboGenericFactory    *factory,
+			       const char              *iid,
+			       PanelAppletCallBackData *data)
+{
+	PanelApplet *applet;
+
+	applet = g_object_new (data->applet_type, NULL);
+
+	panel_applet_setup (applet);
 
 	applet->priv->iid       = g_strdup (iid);
-	applet->priv->closure   = g_closure_ref (user_data);
+	applet->priv->closure   = g_closure_ref (data->closure);
 
 	return BONOBO_OBJECT (applet->priv->control);
 }
@@ -915,19 +1145,24 @@ panel_applet_factory_callback (BonoboGenericFactory *factory,
  * Return value: 0 on success, 1 on failure.
  */
 int
-panel_applet_factory_main_closure (const gchar                 *iid,
-				   GClosure                    *closure)
+panel_applet_factory_main_closure (const gchar *iid,
+				   GType        applet_type,
+				   GClosure    *closure)
 {
-	int                 retval;
+	PanelAppletCallBackData *data;
+	int                      retval;
 
 	g_return_val_if_fail (iid != NULL, 1);
 	g_return_val_if_fail (closure != NULL, 1);
 
+	g_assert (g_type_is_a (applet_type, PANEL_TYPE_APPLET));
+
 	closure = bonobo_closure_store (closure, panel_applet_marshal_BOOLEAN__STRING);
 
-	retval = bonobo_generic_factory_main (iid, panel_applet_factory_callback, closure);
-
-	g_closure_unref (closure);
+	data = panel_applet_callback_data_new (applet_type, closure);
+	retval = bonobo_generic_factory_main (
+			iid, (BonoboFactoryCallback) panel_applet_factory_callback, data);
+	panel_applet_callback_data_free (data);
 
 	return retval;
 }
@@ -945,6 +1180,7 @@ panel_applet_factory_main_closure (const gchar                 *iid,
  */
 int
 panel_applet_factory_main (const gchar                 *iid,
+			   GType                        applet_type,
 			   PanelAppletFactoryCallback   callback,
 			   gpointer                     data)
 {
@@ -953,37 +1189,40 @@ panel_applet_factory_main (const gchar                 *iid,
 	g_return_val_if_fail (iid != NULL, 1);
 	g_return_val_if_fail (callback != NULL, 1);
 
-
 	closure = g_cclosure_new (G_CALLBACK (callback), data, NULL);
 
-	return panel_applet_factory_main_closure (iid, closure);
+	return panel_applet_factory_main_closure (iid, applet_type, closure);
 }
 
 Bonobo_Unknown
-panel_applet_shlib_factory_closure (const char                 *iid,
-				    PortableServer_POA          poa,
-				    gpointer                    impl_ptr,
-				    GClosure                   *closure,
-				    CORBA_Environment          *ev)
+panel_applet_shlib_factory_closure (const char         *iid,
+				    GType               applet_type,
+				    PortableServer_POA  poa,
+				    gpointer            impl_ptr,
+				    GClosure           *closure,
+				    CORBA_Environment  *ev)
 {
 	BonoboShlibFactory *factory;
 
 	g_return_val_if_fail (iid != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail (closure != NULL, CORBA_OBJECT_NIL);
 
+	g_assert (g_type_is_a (applet_type, PANEL_TYPE_APPLET));
+
 	closure = bonobo_closure_store (closure, panel_applet_marshal_BOOLEAN__STRING);
        
-	factory = bonobo_shlib_factory_new_closure
-		(iid, poa, impl_ptr,
-		 g_cclosure_new (G_CALLBACK (panel_applet_factory_callback),
-				 closure,
-				 (GClosureNotify)g_closure_unref));
+	factory = bonobo_shlib_factory_new_closure (
+			iid, poa, impl_ptr,
+			g_cclosure_new (G_CALLBACK (panel_applet_factory_callback),
+					panel_applet_callback_data_new (applet_type, closure),
+					(GClosureNotify) panel_applet_callback_data_free));
 
         return CORBA_Object_duplicate (BONOBO_OBJREF (factory), ev);
 }
 
 Bonobo_Unknown
 panel_applet_shlib_factory (const char                 *iid,
+			    GType                       applet_type,
 			    PortableServer_POA          poa,
 			    gpointer                    impl_ptr,
 			    PanelAppletFactoryCallback  callback,
@@ -993,9 +1232,9 @@ panel_applet_shlib_factory (const char                 *iid,
 	g_return_val_if_fail (iid != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail (callback != NULL, CORBA_OBJECT_NIL);
 
-	return panel_applet_shlib_factory_closure
-		(iid, poa, impl_ptr,
-		 g_cclosure_new (G_CALLBACK (callback),
-				 user_data, NULL),
-		 ev);
+	return panel_applet_shlib_factory_closure (
+			iid, applet_type, poa, impl_ptr,
+			g_cclosure_new (G_CALLBACK (callback),
+					user_data, NULL),
+			ev);
 }
