@@ -40,6 +40,7 @@
 #undef LAUNCHER_DEBUG
 
 static void properties_apply (Launcher *launcher);
+static void launcher_save    (Launcher *launcher);
 
 extern GtkTooltips *panel_tooltips;
 
@@ -107,7 +108,8 @@ launch (Launcher *launcher)
 					   &error);
 		if (error != NULL) {
 			panel_error_dialog ("cannot_launch_icon",
-					    _("Cannot launch icon\n%s"),
+					    _("<b>Cannot launch icon</b>\n\n"
+					      "Details: %s"),
 					    error->message);
 			g_clear_error (&error);
 		}
@@ -358,7 +360,6 @@ create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 	/* Icon/Text will be setup later */
 	launcher->button = button_widget_new (NULL /* icon */,
 					      -1,
-					      LAUNCHER_POBJECT,
 					      FALSE,
 					      PANEL_ORIENT_UP,
 					      NULL /* text */);
@@ -420,7 +421,6 @@ setup_button (Launcher *launcher)
 	const char *comment;
 	const char *name;
 	const char *docpath;
-	char *real_docpath;
 	char *str;
 	char *icon;
 	
@@ -450,6 +450,7 @@ setup_button (Launcher *launcher)
 					icon, -1))
 		button_widget_set_pixmap (BUTTON_WIDGET (launcher->button),
 					  default_app_pixmap, -1);
+	g_free (icon);
 
 	/* Setup help */
 	docpath = gnome_desktop_item_get_string (launcher->ditem,
@@ -457,18 +458,15 @@ setup_button (Launcher *launcher)
 
 	panel_applet_remove_callback (launcher->info, "help_on_app");
 
-	real_docpath = panel_gnome_kde_help_path (docpath);
-	if (real_docpath != NULL) {
+	if (docpath != NULL) {
 		char *title;
 
-		g_free (real_docpath);
-
 		title = g_strdup_printf (_("Help on %s Application"), name);
 
 		panel_applet_add_callback (launcher->info,
 					   "help_on_app",
-					    GTK_STOCK_HELP,
-					    title);
+					   GTK_STOCK_HELP,
+					   title);
 		g_free (title);
 	}
 }
@@ -492,6 +490,8 @@ properties_apply (Launcher *launcher)
 	gnome_desktop_item_set_location (launcher->ditem, location);
 	g_free (location);
 
+	launcher_save (launcher);
+
 	/* Setup the button look */
 	setup_button (launcher);
 }
@@ -597,22 +597,24 @@ launcher_properties (Launcher *launcher)
 }
 
 void
-launcher_save_to_gconf (Launcher  *launcher,
+launcher_save_to_gconf (Launcher   *launcher,
 			const char *gconf_key)
 {
-	GConfClient *client;
-	char        *profile;
-	char        *temp_key;
-	const char  *location;
-
-	client  = panel_gconf_get_client ();
-	profile = session_get_current_profile ();
+	const char *location;
 
 	location = gnome_desktop_item_get_location (launcher->ditem);
+	if (location) {
+		GConfClient *client;
+		const char  *profile;
+		char        *temp_key;
 
-	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "launcher-location");
-	gconf_client_set_string (client, temp_key, location, NULL);
-	g_free (temp_key);
+		client  = panel_gconf_get_client ();
+		profile = session_get_current_profile ();
+
+		temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "launcher-location");
+		gconf_client_set_string (client, temp_key, location, NULL);
+		g_free (temp_key);
+	}
 }
 
 void
@@ -858,6 +860,25 @@ load_launcher_applet (const char  *params,
 	return load_launcher_applet_full (params, NULL, panel, pos, exactpos, gconf_key);
 }
 
+static char *
+launcher_file_name (const char *location)
+{
+	char *tmp, *retval;
+
+	g_return_val_if_fail (location!= NULL, NULL);
+
+	tmp = gnome_util_home_file (PANEL_CONFIG_PATH "launchers");
+	/* Make sure the launcher directory exists */
+	if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
+		panel_ensure_dir (tmp);
+	}
+
+	retval = g_build_filename (tmp, location, NULL);
+
+	g_free (tmp);
+	return retval;
+}
+
 static char *
 launcher_get_unique_uri (void)
 {
@@ -900,27 +921,7 @@ launcher_get_unique_uri (void)
 	return NULL;
 }
 
-char *
-launcher_file_name (const char *location)
-{
-	char *tmp, *retval;
-
-	g_return_val_if_fail (location!= NULL, NULL);
-
-	tmp = gnome_util_home_file (PANEL_CONFIG_PATH "launchers");
-	/* Make sure the launcher directory exists */
-	if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
-		panel_ensure_dir (tmp);
-	}
-
-	retval = g_build_filename (tmp, location, NULL);
-
-	g_free (tmp);
-	return retval;
-}
-
-
-void
+static void
 launcher_save (Launcher *launcher)
 {
 	GError *error;
@@ -956,6 +957,8 @@ launcher_hoard (Launcher *launcher)
 	gnome_desktop_item_set_location (launcher->ditem, NULL);
 
 	launcher_save (launcher);
+
+	launcher_save_to_gconf (launcher, launcher->info->gconf_key);
 }
 
 Launcher *
